import os, sys
import numpy as np
import scipy as sp
import copy
from tqdm import trange
from scipy.sparse import csr_matrix
from sklearn.model_selection import train_test_split
from sklearn.datasets import load_svmlight_file, dump_svmlight_file
np.random.seed(0)

def gen_full_label_data(X, y, n, repeat, reveal_ratio):
    m_hat = X.shape[0]
    X_rows, X_cols = [], []
    y_rows, y_cols = [], []
    X_out, y_out = [], []
    for i in trange(m_hat*repeat):
        select_flag = np.random.rand(len(y[int(i%m_hat)])) < reveal_ratio
        feats = np.array(y[int(i%m_hat)])[select_flag]
        labels = np.array(y[int(i%m_hat)])[~select_flag]
        X_rows.extend([i]*feats.shape[0])
        X_cols.extend(feats.tolist())
        y_rows.extend([i]*labels.shape[0])
        y_cols.extend(labels.tolist())

    X_out = csr_matrix((np.ones(len(X_rows)), (X_rows, X_cols)), shape=(m_hat*repeat, n))
    y_out = csr_matrix((np.ones(len(y_rows)), (y_rows, y_cols)), shape=(m_hat*repeat, n))
    return X_out, y_out

def dump_svm(X, y, fpath):
    # clear context without labels
    rows = np.unique(y.nonzero()[0])
    X_out, y_out = X[rows, :], y[rows, :]
    rows = np.unique(X_out.nonzero()[0])
    X_out, y_out = X_out[rows, :], y_out[rows, :]
    # write to .svm file
    dump_svmlight_file(X_out, y_out, fpath, multilabel=True)
    return

root = sys.argv[1]
repeat = int(sys.argv[2])
reveal_ratio = float(sys.argv[3])

## merge tr.svm and te.svm
flist = [os.path.join(root, f) for f in ['tr.svm', 'te.svm']]
ff = os.path.join(root, 'all.svm')
read_flag = True
with open(flist[0], 'r') as ftr, open(flist[1], 'r') as fte, open(ff, 'w') as fout:
    for trline in ftr:
        trline = trline.rstrip()
        trlabels, trfeats = trline.split(None, 1)
        if read_flag:
            teline = fte.readline().rstrip()
        if teline:
            telabels, tefeats = teline.split(None, 1)
            if tefeats == trfeats:
                labels = trlabels + ',' + telabels
                labels = sorted([l for l in labels.split(',')], key = lambda x: int(x))
                labels = ','.join(labels)
                read_flag = True
            else:
                labels = trlabels
                labels = sorted([l for l in labels.split(',')], key = lambda x: int(x))
                labels = ','.join(labels)
                read_flag = False 
        else:
            labels = trlabels
            labels = sorted([l for l in labels.split(',')], key = lambda x: int(x))
            labels = ','.join(labels)
            read_flag = False
        if len(labels.split(',')) > 1:
            fout.write('%s %s\n'%(labels, trfeats))
        #else:
        #    print('%s'%trfeats)

## load data and split into s1, s2, s3 with ratio [0.2, 0.6, 0.2]
X, y = load_svmlight_file(ff, multilabel=True)
m = X.shape[0]
with open(os.path.join(root, 'label.svm'), 'r') as f:
   n = sum(1 for _ in f)
X_s2, X_s13, y_s2, y_s13 = train_test_split(X, y, test_size=0.4)
X_s1, X_s3, y_s1, y_s3 = train_test_split(X_s13, y_s13, test_size=0.5)
del X_s13, y_s13

for i in range(1, 4):
    X_s = eval('X_s%d'%i)
    y_s = eval('y_s%d'%i)
    X_s, y_s = gen_full_label_data(X_s, y_s, n, repeat, reveal_ratio)
    dump_svm(X_s, y_s, os.path.join(root, 's%d.svm'%i))
