#!/bin/bash

# params
k=128
t=500
b_ratio=1e-2
net_u='Linear'
net_v='Linear'
optim='General'
slot_num=10
#CGmax=30

# others
log_path="./grid_logs"

task(){
# Set up train command
train_cmd="CUDA_VISIBLE_DEVICES=-1 python ./train.py"
train_cmd="${train_cmd}"
train_cmd="${train_cmd} --v_set data/label.svm"
train_cmd="${train_cmd} --val_set data/va.svm"
train_cmd="${train_cmd} --va_tpk data/va.tpk"
#train_cmd="${train_cmd} --skip_mask va.skm"
train_cmd="${train_cmd} -t ${t} -k ${k}"
train_cmd="${train_cmd} --opt ${optim}"
train_cmd="${train_cmd} --b_ratio ${b_ratio}"
train_cmd="${train_cmd} --net_u ${net_u}"
train_cmd="${train_cmd} --net_v ${net_v}"
#train_cmd="${train_cmd} --CGmax ${CGmax}"

# Print out all parameter pair
# 0: RAIPS, 1: Log RAIPS, 2: BCE, 3: BCE+SQ, 4: POXM, 5: Youtube-TopK, 6: BanditNet
for loss in 0 1 3 4 5 6
do
    for lr in 0.05
    do
        for tn in 10 20 50
        do
            for l in 0.25 1 4 16 64 256
            do
                if [ "$loss" == "0" ] || [ "$loss" == "1" ]; then
                    for zeta in 0 0.01 0.1 1
                    do
                        cmd="${train_cmd} --lr ${lr} --loss ${loss} -tpk_num ${tn} --lambda ${l}"
                        cmd="${cmd} -zeta ${zeta}"
                        cmd="${cmd} --u_set data/tr.svm --R data/tr.Rn --weights data/tr.nzinfo"
                        cmd="${cmd} --log ${log_path}/${loss}/${lr}_${l}_${k}_${tn}_${zeta}.log"
                        echo "${cmd}"
                    done
                elif [ "$loss" == "2" ] || [ "$loss" == "3" ]; then
                    for zeta in 0 0.01 0.1 1
                    do
                        for w in 0.0000009537 0.0000152588 0.0002441406
                        do
                            if [ "$loss" == "2" ]; then
                                cmd="${train_cmd} --lr ${lr} --loss ${loss} -tpk_num ${tn} --lambda ${l}"
                                cmd="${cmd} -zeta ${zeta}"
                                cmd="${cmd} --omega ${w}"
                                cmd="${cmd} --u_set data/tr.svm --R data/tr.Rn --weights data/tr.nzinfo"
                                cmd="${cmd} --log ${log_path}/${loss}/${lr}_${l}_${k}_${tn}_${zeta}_${w}.log"
                                echo "${cmd}"
                            else
                                for r in -1
                                do
                                    cmd="${train_cmd} --lr ${lr} --loss ${loss} -tpk_num ${tn} --lambda ${l}"
                                    cmd="${cmd} -zeta ${zeta}"
                                    cmd="${cmd} --omega ${w}"
                                    cmd="${cmd} -r ${r}"
                                    cmd="${cmd} --u_set data/tr.svm --R data/tr.Rn --weights data/tr.nzinfo"
                                    cmd="${cmd} --log ${log_path}/${loss}/${lr}_${l}_${k}_${tn}_${zeta}_${w}_${r}.log"
                                    echo "${cmd}"
                                done
                            fi
                        done
                    done
                elif [ "$loss" == "4" ] || [ "$loss" == "6" ]; then
                    for bl in 1.2 1.0 0.8 0.6
                    do
                        cmd="${train_cmd} --lr ${lr} --loss ${loss} -tpk_num ${tn} --lambda ${l}"
                        if [ "$loss" == "4" ]; then
                            cmd="${cmd} --R data/tr_$bl.Rn --weights data/tr_$bl.nzinfo"
                        else
                            cmd="${cmd} --R data/tr_$bl.nfo.Rn --weights data/tr_$bl.nfo.nzinfo"
                        fi
                        cmd="${cmd} --u_set data/tr.svm --tr_tpk data/tr.tpk"
                        cmd="${cmd} --log ${log_path}/${loss}/${lr}_${l}_${k}_${tn}_${bl}.log"
                        echo "${cmd}"
                    done
                elif [ "$loss" == "5" ]; then
                    cmd="${train_cmd} --lr ${lr} --loss ${loss} -tpk_num ${tn} --lambda ${l}"
                    cmd="${cmd} -slot_num ${slot_num}"
                    cmd="${cmd} --u_set data/tr.nohist.svm --R data/tr.Rn --weights data/tr.nzinfo"
                    cmd="${cmd} --log ${log_path}/${loss}/${lr}_${l}_${k}_${tn}_${slot_num}.log"
                    echo "${cmd}"
                fi
            done
        done
    done
done
}

# Check command
task
wait

# Run
task | xargs -0 -d '\n' -P 2 -I {} sh -c {}
