#!/bin/bash

# params
k=128
t=50
b_ratio=1e-2
net_u='Linear'
net_v='Linear'
optim='NewtonCG'
lr=0.05
CGmax=30

# others
log_path="./grid_logs.ncg"

task(){
# Set up train command
train_cmd="CUDA_VISIBLE_DEVICES=-1 python ./train.py"
train_cmd="${train_cmd} --u_set data/tr.svm"
train_cmd="${train_cmd} --v_set data/label.svm"
train_cmd="${train_cmd} --val_set data/va.svm"
train_cmd="${train_cmd} --va_tpk data/va.tpk"
#train_cmd="${train_cmd} --skip_mask va.skm"
train_cmd="${train_cmd} -t ${t} -k ${k}"
train_cmd="${train_cmd} --opt ${optim}"
#train_cmd="${train_cmd} --b_ratio ${b_ratio}"
train_cmd="${train_cmd} --net_u ${net_u}"
train_cmd="${train_cmd} --net_v ${net_v}"
train_cmd="${train_cmd} --CGmax ${CGmax}"

# Print out all parameter pair
# 0: RAIPS, 1: Log RAIPS, 2: BCE, 3: BCE+SQ, 4: POXM, 5: Youtube-TopK, 6: BanditNet
for loss in 3
do
    for xi in 0.3 0.5
    do
        for tn in 10
        do
            for l in 0.25 1 4 16 64 256
            do
                if [ "$loss" == "3" ]; then
                    for zeta in 0 0.01 0.1 1
                    do
                        for w in 0.0000009537 0.0000152588 0.0002441406
                        do
                            for r in -1
                            do
                                cmd="${train_cmd} --lr ${lr} --loss ${loss} -tpk_num ${tn} --lambda ${l} --xi ${xi}"
                                cmd="${cmd} -zeta ${zeta}"
                                cmd="${cmd} --omega ${w}"
                                cmd="${cmd} -r ${r}"
                                cmd="${cmd} --R data/tr.Rn --weights data/tr.nzinfo"
                                cmd="${cmd} --log ${log_path}/${loss}/${lr}_${l}_${k}_${tn}_${zeta}_${w}_${r}_${xi}.log"
                                echo "${cmd}"
                            done
                        done
                    done
                fi
            done
        done
    done
done
}

# Check command
task
wait

# Run
task | xargs -0 -d '\n' -P 2 -I {} sh -c {}
