import os, sys
from shutil import copy
import numpy as np
from sklearn.preprocessing import normalize


def get_slot_num(fpath):
    slot_num = 0
    with open(fpath, 'r') as fin:
        for line in fin:
            line = line.strip().split(',')
            tmp_slot_num = max(len(line), slot_num)
            if tmp_slot_num > slot_num:
                slot_num = tmp_slot_num
            else:
                break
    return slot_num

def get_item_num(fpath):
    item_num = 0
    with open(fpath, 'r') as fin:
        for _ in fin:
            item_num += 1
    return item_num

iroot = sys.argv[1]
oroot = sys.argv[2] 
bn_lambda = sys.argv[3]
flag=sys.argv[4]

os.makedirs(oroot, exist_ok=True)

list1 = "te.svm va.svm".split()
list2 = ["tr"]
pos_reward = 1.0
neg_reward = 0.001
item_num = get_item_num(os.path.join(iroot, 'label.svm'))
print(item_num)

if flag == '1':
    copy(os.path.join(iroot, 'label.svm'), oroot)
    for f in list1:
        copy(os.path.join(iroot, f.replace('.svm', '.tpk')), oroot)
        with open(os.path.join(iroot, f), 'r') as fin, \
                open(os.path.join(oroot, f), 'w') as fout:
            for line in fin:
                line = line.strip()
                labels, feats = line.split(maxsplit=1)
                labels = [l.split(':')[0] for l in labels.split(',')]
                labels = ','.join(labels)
                fout.write('%s %s\n'%(labels, feats))

    for f in list2:
        copy(os.path.join(iroot, f+'.tpk'), oroot)
        with open(os.path.join(iroot, f+'.svm'), 'r') as fsin, \
                open(os.path.join(iroot, f+'.plm'), 'r') as fpin, \
                open(os.path.join(oroot, f+'.plm'), 'w') as fpout, \
                open(os.path.join(oroot, f+'.svm'), 'w') as fsout, \
                open(os.path.join(oroot, f+'.nohist.svm'), 'w') as fnhsout, \
                open(os.path.join(oroot, f+'.Rn'), 'w') as frout, \
                open(os.path.join(oroot, f+'.nzinfo'), 'w') as fnout:
            for sline in fsin:
                sline = sline.strip()
                pline = fpin.readline().strip()

                label, nhfeats = sline.split(maxsplit=1)
                label, click, pps = label.split(':')
                hist = ['%d:1'%(int(_h)+item_num) for _h in pline.split(',')][:-1]
                hist.extend(nhfeats.split())
                feats = sorted(hist, key=lambda x: int(x.split(':')[0]))
                feats = ' '.join(feats)

                pline = ','.join(pline.split(',')[:-1])

                if click == '1':
                    Rn = (pos_reward)/float(pps)
                    nz = '%f:%s'%(pos_reward, pps)
                else:
                    Rn = (neg_reward)/float(pps)
                    nz = '%f:%s'%(neg_reward, pps)

                fsout.write('%s %s\n'%(label, feats))
                fnhsout.write('%s %s\n'%(label, nhfeats))
                frout.write('%.16f\n'%(Rn))
                fnout.write('%s\n'%(nz))
                fpout.write('%s\n'%(pline))
else:
    for f in list2:
        slot_num = get_slot_num(os.path.join(iroot, f+'.plm'))
        with open(os.path.join(iroot, f+'.svm'), 'r') as fsin, \
                open(os.path.join(oroot, f+'_%s.Rn'%bn_lambda), 'w') as frout, \
                open(os.path.join(oroot, f+'_%s.nzinfo'%bn_lambda), 'w') as fnout, \
                open(os.path.join(oroot, f+'_%s.nfo.Rn'%bn_lambda), 'w') as ffrout, \
                open(os.path.join(oroot, f+'_%s.nfo.nzinfo'%bn_lambda), 'w') as ffnout:
            _c = 0
            acc_click, acc_pps, slot_pps = [], [], []
            for sline in fsin:
                _c += 1
                sline = sline.strip()

                label, feats = sline.split(maxsplit=1)
                label, click, pps = label.split(':')
                acc_click.append(click)
                slot_pps.append(float(pps))

                if _c == slot_num:
                    acc_pps.append(np.array(slot_pps).prod())
                    _c, slot_pps = 0, []

                # first-order
                if click == '1':
                    Rn = (pos_reward-1.0+float(bn_lambda))/float(pps)
                    nz = '%f:%s'%(pos_reward-1.0+float(bn_lambda), pps)
                else:
                    Rn = (neg_reward-1.0+float(bn_lambda))/float(pps)
                    nz = '%f:%s'%(neg_reward-1.0+float(bn_lambda), pps)

                frout.write('%.16f\n'%(Rn))
                fnout.write('%s\n'%(nz))

            # no first-order
            assert len(acc_click) == slot_num*len(acc_pps)
            acc_pps = normalize(1./np.array(acc_pps).reshape(1, -1), norm='max').ravel() if slot_num > 1 else np.array(acc_pps).ravel()
            for i, click in enumerate(acc_click):
                pps = '%.16f'%(acc_pps[int(i//slot_num)])
                if click == '1':
                    Rn = (pos_reward-1.0+float(bn_lambda))/float(pps)
                    nz = '%f:%s'%(pos_reward-1.0+float(bn_lambda), pps)
                else:
                    Rn = (neg_reward-1.0+float(bn_lambda))/float(pps)
                    nz = '%f:%s'%(neg_reward-1.0+float(bn_lambda), pps)

                ffrout.write('%.16f\n'%(Rn))
                ffnout.write('%s\n'%(nz))
