#!/bin/bash

ds=$1
pos=$2
log_path="./${ds}-slot$pos/best_params"
data_path="./${ds}-slot$pos/data"
b_ratio=1e-2
net_u='Linear'
net_v='Linear'
optim='General'
#CGmax=30


task(){
# Set up train command
for params in `cat ./${ds}-slot$pos/params`
do
	# params
	loss=`echo ${params} | cut -d'_' -f1`
	t=`echo ${params} | cut -d'_' -f2`
	lr=`echo ${params} | cut -d'_' -f3`
	l=`echo ${params} | cut -d'_' -f4`
	k=`echo ${params} | cut -d'_' -f5`
	tn=`echo ${params} | cut -d'_' -f6`
	zeta=`echo ${params} | cut -d'_' -f7`
	bl=`echo ${params} | cut -d'_' -f7`
	sn=`echo ${params} | cut -d'_' -f7`
	w=`echo ${params} | cut -d'_' -f8`
	r=`echo ${params} | cut -d'_' -f9`
	t=$(echo "${t}" | bc -l)

	train_cmd="CUDA_VISIBLE_DEVICES=-1 python ./train.py"
	train_cmd="${train_cmd}"
	train_cmd="${train_cmd} --v_set $data_path/label.svm"
	train_cmd="${train_cmd} --val_set $data_path/te.svm"
	train_cmd="${train_cmd} --va_tpk $data_path/te.tpk"
	#train_cmd="${train_cmd} --skip_mask te.skm"
	train_cmd="${train_cmd} --opt ${optim}"
	train_cmd="${train_cmd} --b_ratio ${b_ratio}"
	train_cmd="${train_cmd} --net_u ${net_u}"
	train_cmd="${train_cmd} --net_v ${net_v}"
	#train_cmd="${train_cmd} --CGmax ${CGmax}"
	
	# Print out all parameter pair
	cmd="${train_cmd} --loss ${loss} -t ${t} --lr ${lr} --lambda ${l} -k ${k} -tpk_num ${tn}"
	if [ "$loss" == "0" ] || [ "$loss" == "1" ]; then
	    cmd="${cmd} -zeta ${zeta}"
	    cmd="${cmd} --u_set $data_path/tr.svm --R $data_path/tr.Rn --weights $data_path/tr.nzinfo"
	    cmd="${cmd} --log ${log_path}/${loss}_${lr}_${l}_${k}_${tn}_${zeta}_${t}.log"
	    cmd="${cmd} --model ${log_path}/${loss}_model.cpkt"
		echo "${cmd}"
	elif [ "$loss" == "2" ]; then
	    cmd="${cmd} -zeta ${zeta}"
	    cmd="${cmd} --omega ${w}"
	    cmd="${cmd} --u_set $data_path/tr.svm --R $data_path/tr.Rn --weights $data_path/tr.nzinfo"
	    cmd="${cmd} --log ${log_path}/${loss}_${lr}_${l}_${k}_${tn}_${zeta}_${w}_${t}.log"
	    cmd="${cmd} --model ${log_path}/${loss}_model.cpkt"
		echo "${cmd}"
	elif [ "$loss" == "3" ]; then
	    cmd="${cmd} -zeta ${zeta}"
	    cmd="${cmd} --omega ${w}"
	    cmd="${cmd} -r ${r}"
	    cmd="${cmd} --u_set $data_path/tr.svm --R $data_path/tr.Rn --weights $data_path/tr.nzinfo"
	    cmd="${cmd} --log ${log_path}/${loss}_${lr}_${l}_${k}_${tn}_${zeta}_${w}_${r}_${t}.log"
	    cmd="${cmd} --model ${log_path}/${loss}_model.cpkt"
		echo "${cmd}"
	elif [ "$loss" == "5" ]; then
	    cmd="${cmd} -slot_num ${sn}"
	    cmd="${cmd} --u_set $data_path/tr.nohist.svm --R $data_path/tr.Rn --weights $data_path/tr.nzinfo"
	    cmd="${cmd} --log ${log_path}/${loss}_${lr}_${l}_${k}_${tn}_${sn}_${t}.log"
	    cmd="${cmd} --model ${log_path}/${loss}_model.cpkt"
		echo "${cmd}"
	elif [ "$loss" == "4" ]; then
	    cmd="${cmd} --R $data_path/tr_$bl.Rn --weights $data_path/tr_$bl.nzinfo"
	    cmd="${cmd} --u_set $data_path/tr.svm --tr_tpk $data_path/tr.tpk"
	    cmd="${cmd} --log ${log_path}/${loss}_${lr}_${l}_${k}_${tn}_${bl}_${t}.log"
	    cmd="${cmd} --model ${log_path}/${loss}_model.cpkt"
		echo "${cmd}"
	elif [ "$loss" == "6" ]; then
	    cmd="${cmd} --R $data_path/tr_$bl.nfo.Rn --weights $data_path/tr_$bl.nfo.nzinfo"
	    cmd="${cmd} --u_set $data_path/tr.svm --tr_tpk $data_path/tr.tpk"
	    cmd="${cmd} --log ${log_path}/${loss}_${lr}_${l}_${k}_${tn}_${bl}_${t}.log"
	    cmd="${cmd} --model ${log_path}/${loss}_model.cpkt"
		echo "${cmd}"
	fi
done
}


# Check command
task
wait


# Run
task | xargs -0 -d '\n' -P 2 -I {} sh -c {}
