from utilities import read_data, ConfigClass, gen_skip_mask
import numpy as np
import tensorflow as tf
tf.compat.v1.disable_eager_execution()
import time
import math
import argparse

from net.net import MVN
from newton_cg import NewtonCG
from sogram import Sogram
from minibatch import Minibatch
from general import General
import pdb

def parse_args():
    parser = argparse.ArgumentParser(description='Newton method on DNN')
    parser.add_argument('--C', dest='C',
                      help='regularization term, or so-called weight decay where'+\
                            'weight_decay = lr/(C*num_of_samples) in this implementation' ,
                      default=0.01, type=float)

    # Newton method arguments
    parser.add_argument('--omega', dest='omega',
                      help='the balance for grammian part',
                      default=0.1, type=float)
    parser.add_argument('--GNsize', dest='GNsize',
                      help='number of samples for estimating Gauss-Newton matrix',
                      default=4096, type=int)
    parser.add_argument('-t', dest='iter_max',
                      help='the maximal number of Newton iterations',
                      default=100, type=int)
    parser.add_argument('--xi', dest='xi',
                      help='the tolerance in the relative stopping condition for CG',
                      default=0.1, type=float)
    parser.add_argument('--eta', dest='eta',
                      help='the parameter for the line search stopping condition',
                      default=0.0001, type=float)
    parser.add_argument('--CGmax', dest='CGmax',
                      help='the maximal number of CG iterations',
                      default=20, type=int)
    parser.add_argument('--lambda', dest='_lambda',
                      help='the initial lambda for the LM method',
                      default=1, type=float)
    parser.add_argument('--alpha', dest='alpha',
                      help='fixed step size (conflicting from_prev)',
                      default=0, type=float)
    parser.add_argument('--gramian_alpha', dest='gramian_alpha',
                      help='alpha for gramian',
                      default=0.1, type=float)
    parser.add_argument('--b_ratio', dest='b_ratio',
                      help='ratio of nnz for each batch',
                      default=1, type=float)
    parser.add_argument('--from_prev', dest='from_prev',
                      help='start from previous step size (conflicting alpha)',
                      default=False, type=bool)
    parser.add_argument('--thres', dest='thres',
                      help='threshold for doubling alpha',
                      default=0.9, type=float)

    # SGD arguments
    parser.add_argument('--lr', dest='lr',
                      help='learning rate',
                      default=0.01, type=float)

    # Model training arguments

    parser.add_argument('--bsize', dest='bsize',
                      help='batch size to evaluate stochastic gradient, Gv, etc. Since the sampled data \
                      for computing Gauss-Newton matrix and etc. might not fit into memeory \
                      for one time, we will split the data into several segements and average\
                      over them.',
                      default=2**22, type=int)

    parser.add_argument('-k', dest='k',
                      help='output size for P and Q',
                      default=64, type=int)
    parser.add_argument('-k1', dest='k1',
                      help='output size for tP and tQ',
                      default=4, type=int)

    parser.add_argument('-r', dest='r',
                      help='defalut imputed label',
                      default=0, type=float)
    parser.add_argument('-zeta', dest='zeta',
                      help='weight for trust region term',
                      default=0, type=float)
    parser.add_argument('-tpk_num', dest='tpk_num',
                      help='the number of top-ranking items reserved for POXM',
                      default=10, type=int)
    parser.add_argument('-slot_num', dest='slot_num',
                      help='the number of slot',
                      default=1, type=int)

    parser.add_argument('--net_u', dest='net_u',
                      help='architecture for instances',
                      default="WideNN", type=str)
    parser.add_argument('--net_v', dest='net_v',
                      help='architecture for label',
                      default="WideNN", type=str)

    parser.add_argument('--u_set', dest='u_set',
                      help='provide the directory of .mat file for training',
                      default='data/bibtex.tr', type=str)
    parser.add_argument('--weights', dest='weights',
                      help='provide the path of weight file',
                      default=None, type=str)
    parser.add_argument('--R', dest='R',
                      help='provide the path of R file',
                      default=None, type=str)
    parser.add_argument('--tr_tpk', dest='tr_tpk',
                      help='provide the path of tpk file for training',
                      default=None, type=str)
    parser.add_argument('--pl', dest='pl_path',
                      help='provide the path of past actions file for training',
                      default=None, type=str)
    parser.add_argument('--va_tpk', dest='va_tpk',
                      help='provide the path of tpk file for validation',
                      default=None, type=str)
    parser.add_argument('--v_set', dest='v_set',
                      help='provide the directory of .mat file for training',
                      default='data/label.svm', type=str)
    parser.add_argument('--val_set', dest='val_set',
                      help='provide the directory of .mat file for validation',
                      default=None, type=str)
    parser.add_argument('--skip_mask', dest='skip_mask',
                      help='provide the file masking items in training set for validation',
                      default=None, type=str)
    parser.add_argument('--model', dest='model_file',
                      help='model saving address',
                      default='./saved_model/model.ckpt', type=str)
    parser.add_argument('--loss', dest='loss_type',
            help='0:IPS, 1:log-IPS, 2:BCE, 3:BCE-SQ, 4:POXM, 5:TopK-Reinforce, 6:BanditNet, 7:Value-Based',
                      default=0, type=int)
    parser.add_argument('--log', dest='log_file',
                      help='log saving directory',
                      default='./running_log/logger.log', type=str)
    parser.add_argument('--save_pq', dest='save_pq',
                      help='storing .npy file for P, Q',
                      action='store_true')
    parser.add_argument('--screen_log_only', dest='screen_log_only',
                      help='screen printing running log instead of storing it',
                      action='store_true')
    parser.add_argument('--optim', '-optim',
                      help='which optimizer to use: Sogram, Minibatch, or NewtonCG, General',
                      default='NewtonCG', type=str)
    parser.add_argument('--seed', dest='seed', help='a nonnegative integer for \
                        reproducibility', default=8964, type=int)
    parser.add_argument('--profile', dest='profile',
                      help='whether to show time profiling',
                      default=0, type=int)
    parser.add_argument('--loader', dest='loader',
                      help='which type of data loader to use: sparse or dense',
                      default='sparse', type=str)
    parser.add_argument('--precision', dest='precision',
                      help='floating point precision: 32, or 64',
                      default=64, type=int)
    args = parser.parse_args()
    return args

args = parse_args()

def init_model(param, dtype):
    init_ops = []
    for p in param:
        if 'kernel' in p.name:
            shape = list(map(int, p.get_shape()))
            initializer = tf.keras.initializers.GlorotUniform()
            weight = initializer(shape, dtype=dtype)
            opt = tf.compat.v1.assign(p, weight)
        elif 'bias' in p.name:
            zeros = np.zeros(p.shape)
            opt = tf.compat.v1.assign(p, zeros)
        init_ops.append(opt)
    return tf.group(*init_ops)


def main():
    precision = 'float' + str(args.precision)
    dtype = getattr(tf, precision)
    dtype_np = getattr(np, precision)
    trainL, trainR, validL, validR = read_data(args.loader, args.u_set, args.v_set, args.val_set, dtype_np)
    Ytr = trainL.label
    Yte = validL.label
    
    if args.val_set is None:
        print('No validation set is provided. Will output model at the last iteration.')

    if args.R is not None:
        R = np.loadtxt(args.R, dtype=dtype_np).ravel()
    else:
        R = np.ones([Ytr.m], dtype=dtype_np)
    if args.weights is not None:
        Ytr.read_weights(args.weights, dtype_np, args.loss_type==7)
    if args.tr_tpk is not None:
        Ytr.read_tpk(args.tr_tpk, args.tpk_num, False, dtype_np)

    if args.va_tpk is not None:
        Yte.read_tpk(args.va_tpk, args.tpk_num, True, dtype_np)

    Ytr.read_pl_mask(args.pl_path, args.loss_type)
    skip_mask = gen_skip_mask(Ytr.m, Ytr.n, args.skip_mask)

    config = ConfigClass(args, Ytr, skip_mask, R)
    if isinstance(config.seed, int):
        tf.compat.v1.random.set_random_seed(config.seed)
        np.random.seed(config.seed)
    if config.net_u in ('Linear', 'ThinNN', 'WideNN'):
        u,v,p,q = MVN(config.net_u, config.net_v, trainL.feat, trainR.feat, config.k, dtype)
    else:
        raise ValueError('Unrecognized training model')

    network = (u,v,p,q)

    sess_config = tf.compat.v1.ConfigProto(intra_op_parallelism_threads=0,
            inter_op_parallelism_threads=0)
    sess_config.gpu_options.allow_growth = True

    with tf.compat.v1.Session(config=sess_config) as sess:

        param = tf.compat.v1.trainable_variables()
        saver = tf.compat.v1.train.Saver(var_list=param)

        size = np.sum([np.prod(v.get_shape().as_list()) for v in param])
        print('Model size: {}'.format(size))

        Solver = eval(config.optim)
        solver = Solver(config, sess, network)

        sess.run(tf.compat.v1.global_variables_initializer())
        print('-------------- initializing network by methods in He et al. (2015) --------------')
        param = tf.compat.v1.trainable_variables()
        sess.run(init_model(param, dtype))
        solver.train(trainL, trainR, validL, validR, saver)

if __name__ == '__main__':
    main()
