import tensorflow.compat.v1 as tf
tf.disable_v2_behavior()
import math
import pdb
from tensorflow.python.client import device_lib

import numpy as np

class SparseLinear(tf.keras.layers.Layer):
    """y = w.x + b"""

    def __init__(self, input_dim, units, dtype, has_bias=False):
        super(SparseLinear, self).__init__()
        self.has_bias = has_bias
        w_init = tf.random_normal_initializer()
        self.w = tf.Variable(
            initial_value=w_init(shape=(input_dim, units), dtype=dtype),
            trainable=True, name="kernel"
        )
        if has_bias:
            b_init = tf.zeros_initializer()
            self.b = tf.Variable(
                initial_value=b_init(shape=(units,), dtype=dtype), trainable=True, name='bias'
            )

    def call(self, inputs):
        if self.has_bias:
            return tf.sparse.sparse_dense_matmul(inputs, self.w) + self.b
        else:
            return tf.sparse.sparse_dense_matmul(inputs, self.w)

class _DeepNN(tf.keras.layers.Layer):
    def __init__(self, input_dim, units, dtype, net, has_bias=False):
        super(_DeepNN, self).__init__()
        self.has_bias = has_bias
        w_init = tf.random_normal_initializer()
        b_init = tf.zeros_initializer()
        if net == 'ThinNN':
            neurons = [input_dim, 256, units]
        elif net == 'WideNN':
            neurons = [input_dim, 256, 256, units]
        else:
            raise ValueError('Unknown Net!')
        self.kernels, self.bias = [], []
        self.neurons = neurons
        for i in range(len(neurons)-1):
            self.kernels.append(
                tf.Variable(
                initial_value=w_init(shape=(neurons[i], neurons[i+1]), dtype=dtype),
                trainable=True, name="kernel_"+str(i))
            )
            if self.has_bias:
                self.bias.append(
                    tf.Variable(
                    initial_value=b_init(shape=(neurons[i+1],), dtype=dtype), trainable=True, name='bias_'+str(i))
                )
    
    def call(self, inputs):
        x = inputs 
        for i in range(len(self.neurons)-1):
            if i == 0:
                x = tf.sparse.sparse_dense_matmul(x, self.kernels[i])
            else:
                x = tf.matmul(x, self.kernels[i])
            if self.has_bias:
                x = x + self.bias[i]
            if i != len(self.neurons)-2:
                x = tf.keras.activations.elu(x)
        return x


def Linear(u, D, k, dtype, scope_name, reuse=False):
    _NUM_INPUTS = D
    _NUM_OUTPUTS = k
    with tf.variable_scope(scope_name, reuse=reuse) as scope:
        outputs = SparseLinear(_NUM_INPUTS, _NUM_OUTPUTS, dtype)(u)
    return outputs

def ThinNN(u, D, k, dtype, scope_name, reuse=False):
    _NUM_INPUTS = D
    _NUM_OUTPUTS = k
    with tf.variable_scope(scope_name, reuse=reuse) as scope:
        outputs = _DeepNN(_NUM_INPUTS, _NUM_OUTPUTS, dtype, net='ThinNN')(u)
    return outputs

def WideNN(u, D, k, dtype, scope_name, reuse=False):
    _NUM_INPUTS = D
    _NUM_OUTPUTS = k
    with tf.variable_scope(scope_name, reuse=reuse) as scope:
        outputs = _DeepNN(_NUM_INPUTS, _NUM_OUTPUTS, dtype, net='WideNN')(u)
    return outputs

def MVN(net_u, net_v, U, V, k, dtype):

    with tf.name_scope('main_params'):
        #TODO investigate the input format of sparse feature vector
        u = tf.sparse_placeholder(dtype, name='input_of_net_u')
        v = tf.sparse_placeholder(dtype, name='input_of_net_v')

    # call MVN structure according to string net
    if net_u in ('Linear', 'ThinNN', 'WideNN'):
        D_u = U.shape[1]
        D_v = V.shape[1]

        p = globals()[net_u](u, D_u, k, dtype, 'u_model')
        p = tf.identity(p, name='output_of_net_u')

        q = globals()[net_v](v, D_v, k, dtype, 'v_model')
        q = tf.identity(q, name='output_of_net_v')
    # TODO: add conv net

    return (u, v, p, q)
